package EarthquakeFrame;
import java.util.List;
import java.util.ArrayList;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.table.*;

import EarthquakeLib.Earthquake;
import EarthquakeLib.EarthquakeImporter;
import EarthquakeLib.EarthquakeManager;

import java.io.*;

//Kyoung Shin Park (2017 Fall)

public class EarthquakeTableFrame extends JFrame {

	//EarthquakeMapViewer map;
	EarthquakeTableModel model = null;
	
	public EarthquakeTableFrame(EarthquakeManager manager) {
		super("EarthquakeTableFrame");

		//map = new EarthquakeMapViewer(quakeList);
		//map.setVisible(true);

		this.model = new EarthquakeTableModel(manager);
		JTable table = new JTable(this.model);
		table.setAutoCreateRowSorter(false);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setIntercellSpacing(new Dimension(10,5));
		table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

			@Override
			public void valueChanged(ListSelectionEvent e) {
				ListSelectionModel lsm = (ListSelectionModel)e.getSource();
				if (!e.getValueIsAdjusting() && !lsm.isSelectionEmpty()) {
					int index = table.getSelectedRow();
					if (index >= 0 && index < manager.size()) {
						model.removeRow(index);
						model.fireTableDataChanged(); // repaint JTable
					}
				}
			}
			
		});
		
		JTableHeader header = table.getTableHeader();
	    header.setUpdateTableInRealTime(true);
	    header.addMouseListener(model.new ColumnListener(table));
	    header.setReorderingAllowed(true);
	    
		JScrollPane scrollPane = new JScrollPane(table);
		this.add(scrollPane, BorderLayout.CENTER);

		// menu
		JMenuBar menuBar = new JMenuBar();

		// File Menu, F - Mnemonic
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);
		menuBar.add(fileMenu);
		
		// File->Open, O - Mnemonic
		JMenuItem openMenuItem = new JMenuItem("Open", KeyEvent.VK_O);
		fileMenu.add(openMenuItem);
		this.setJMenuBar(menuBar);
		openMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
		        System.out.println("OPEN");
		        String selectedFilename = openFileDialog();
		        if (selectedFilename != null) {
		        	manager.loadCSV(selectedFilename);
		        	model.fireTableDataChanged(); // repaint JTable
		        }
			}
		});
		
		// File->Save, S - Mnemonic
		JMenuItem saveMenuItem = new JMenuItem("Save", KeyEvent.VK_S);
		fileMenu.add(saveMenuItem);
		this.setJMenuBar(menuBar);
		saveMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
		        System.out.println("SAVE");
		        String selectedFilename = saveFileDialog();
		        if (selectedFilename != null) {
		        	manager.writeCSV(selectedFilename);
		        }
			}
		});
		
		// File->Exit, E - Mnemonic
		JMenuItem exitMenuItem = new JMenuItem("Exit", KeyEvent.VK_E);
		fileMenu.add(exitMenuItem);
		this.setJMenuBar(menuBar);
		exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				manager.writeCSV("out.csv");
				System.exit(0);
			}
		});

		// Quake Menu, Q - Mnemonic
		JMenu quakeMenu = new JMenu("Quake");
		quakeMenu.setMnemonic(KeyEvent.VK_Q);
		menuBar.add(quakeMenu);
		
		// Quake->Find, T - Mnemonic
		JMenuItem findMenuItem = new JMenuItem("Find", KeyEvent.VK_T);
		quakeMenu.add(findMenuItem);
		this.setJMenuBar(menuBar);
		findMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				FindDialog findDialog = new FindDialog();
				findDialog.setVisible(true);
				if (findDialog.getReturnValue() == FindDialog.OK) {
					System.out.println("FIND");
					double[][] range = findDialog.getRange();
					List<Earthquake> qList = manager.findAllIf(q -> q.Magnitude > range[3][0] && q.Magnitude < range[3][1]);
					qList.forEach(q -> System.out.println(q));
				}
			}
		});

		this.setSize(600, 650);
		this.setVisible(true);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	public String openFileDialog() {
		JFileChooser fileChooser = new JFileChooser();
        int returnValue = fileChooser.showOpenDialog(null);
        if (returnValue == JFileChooser.APPROVE_OPTION) {
          System.out.println("openFile");
          String selectedFilename = fileChooser.getSelectedFile().getAbsolutePath().toString();
          System.out.println(selectedFilename);
          return selectedFilename;
        }
        else {
            System.out.println("file not opened");
        	return null;
        }
	}
	

	public String saveFileDialog() {
		JFileChooser fileChooser = new JFileChooser();
        int returnValue = fileChooser.showSaveDialog(null);
        if (returnValue == JFileChooser.APPROVE_OPTION) {
          System.out.println("saveFile");
          String selectedFilename = fileChooser.getSelectedFile().getAbsolutePath().toString();
          System.out.println(selectedFilename);
          return selectedFilename;
        }
        else {
            System.out.println("file not selected");
        	return null;
        }
	}

}