package EarthquakeFrame;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.*;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.table.*;

import EarthquakeLib.Earthquake;
import EarthquakeLib.EarthquakeManager;

public class EarthquakeTableModel extends AbstractTableModel {

	private EarthquakeManager manager;
	private String[] columnNames = Earthquake.columnNames();

	public EarthquakeTableModel(EarthquakeManager manager) {
		this.manager = manager;
	}

	@Override
	public int getRowCount() {
		return this.manager.size();
	}

	@Override
	public int getColumnCount() {
		return columnNames.length;
	}

	@Override
	public String getColumnName(int columnIndex) {
		return columnNames[columnIndex];
	}

	// For sorting, the important point is the method getColumnClass()
	@Override
	public Class<?> getColumnClass(int columnIndex) {
		if (this.manager.isEmpty()) {
			return Object.class;
        }
        return getValueAt(0, columnIndex).getClass();
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		Earthquake q = this.manager.get(rowIndex);
		Object value = null;
		switch (columnIndex) {
		case 0:
			value = q.Time;
			break;
		case 1:
			value = q.Latitude;
			break;
		case 2:
			value = q.Longitude;
			break;
		case 3:
			value = q.Depth;
			break;
		case 4:
			value = q.Magnitude;
			break;
		case 5:
			value = q.MagType;
			break;
		case 6:
			value = q.ID;
			break;
		case 7:
			value = q.Place;
			break;
		}
		return value;
	}
		
	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		Earthquake q = this.manager.get(rowIndex);
		switch (columnIndex) {
		case 0:
			q.Time = (Date)value;
			break;
		case 1:
			q.Latitude = (Double)value;
			break;
		case 2:
			q.Longitude = (Double)value;
			break;
		case 3:
			q.Depth = (Double)value;
			break;
		case 4:
			q.Magnitude = (Double)value;
			break;
		case 5:
			q.MagType = (String)value;
			break;
		case 6:
			q.ID = (String)value;
			break;
		case 7:
			q.Place = (String)value;
			break;
		default:
            throw new IllegalArgumentException("Invalid column index");
		}
		fireTableDataChanged();
	}

	public void addRow(Earthquake q) {
		System.out.println("addRow: " + q + " RowCount=" + getRowCount());
		this.manager.add(q);       
		fireTableRowsInserted(this.manager.size() - 1, this.manager.size() - 1);
	}
         
	public void removeRow(int rowIndex) {
		if (rowIndex >=0 && rowIndex < getRowCount()) {
			System.out.println("removeRow rowIndex=" + rowIndex);
			fireTableRowsDeleted(rowIndex, rowIndex);
			this.manager.removeAt(rowIndex);
        }
	}
		
	class ColumnListener extends MouseAdapter {
		private JTable table;
		public ColumnListener(JTable table) {
			this.table = table;
		}

		public void mouseClicked(MouseEvent e) {
			TableColumnModel colModel = table.getColumnModel();
			int columnModelIndex = colModel.getColumnIndexAtX(e.getX());
			int modelIndex = colModel.getColumn(columnModelIndex).getModelIndex();

			if (modelIndex < 0)
				return;

			for (int i = 0; i < getColumnCount(); i++) { 
				TableColumn column = colModel.getColumn(i);
				column.setHeaderValue(getColumnName(column.getModelIndex()));
			}
			table.getTableHeader().repaint();

			manager.sort(columnModelIndex);
			
			table.tableChanged(new TableModelEvent(EarthquakeTableModel.this));
			table.repaint();
		}
	}
}   
