package EarthquakeLib;
import java.awt.Color;
import java.awt.image.ColorModel;
import java.util.*;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;

public class Earthquake implements Comparable<Earthquake> {

    private static final String DATE_FORMAT_PATTERN = "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'";
	private static final SimpleDateFormat formatter = new SimpleDateFormat(DATE_FORMAT_PATTERN);
    
	public Date Time;
	public double Latitude;
	public double Longitude;
	public double Depth;
	public double Magnitude;
	public String MagType;
	public int Nst;
	public double Gap;
	public double Dmin;
	public double Rms;
	public String Net;
	public String ID;
	public Date Updated;
	public String Place;
	public String Type;
	public double HorizontalError;
	public double DepthError;
	public double MagError;
	public int MagNst;
	public String Status;
	public String LocationSource;
	public String MagSource;
	
	public Color MagnitudeColor;

	public Earthquake() {
		this(new Date(), 0.0, 0.0, 0.0, 0.0, "", "", "");
	}
	
	public Earthquake(Date time, double lat, double lon, double depth, double mag, String magType, String id, String place) {
        this.Time = time;
        this.Latitude = lat;
        this.Longitude = lon;
        this.Depth = depth;
        this.Magnitude = mag;
        this.MagType = magType;
        this.ID = id;
        this.Place = place;
        this.MagnitudeColor = computeARGBColor();
	}
	
    public Earthquake(Date time, double lat, double lon, double depth, double mag, String magType, int nst, double gap, double dmin, double rms, String net, String id, Date updated, String place, String type, double he, double de, double me, int magNst, String status, String locSource, String magSource) {
        this.Time = time;
        this.Latitude = lat;
        this.Longitude = lon;
        this.Depth = depth;
        this.Magnitude = mag;
        this.MagType = magType;
        this.Nst = nst;
        this.Gap = gap;
        this.Dmin = dmin;
        this.Rms = rms;
        this.Net = net;
        this.ID = id;
        this.Updated = updated;
        this.Place = place;
        this.Type = type;
        this.HorizontalError = he;
        this.DepthError = de;
        this.MagError = me;
        this.MagNst = magNst;
        this.Status = status;
        this.LocationSource = locSource;
        this.MagSource = magSource;
    }
    // compute color based on magnitude
    public Color computeARGBColor() {
        int r = 0, g = 0, b = 0;
        if (this.Magnitude <= 1.5)
        {
            // BLUE
            r = 0; g = 0; b = 255;
        }
        else if ((this.Magnitude > 1.5) && (this.Magnitude <= 2.75))
        {
            // BLUE->CYAN
            r = 0; g = (int)(255 * (this.Magnitude - 1.5) / 1.25); b = 255;
        }
        else if ((this.Magnitude > 2.75) && (this.Magnitude <= 4.0))
        {
            // CYAN->GREEN
            r = 0; g = 255; b = 255 - (int)(255 * (this.Magnitude - 2.75) / 1.25);
        }
        else if ((this.Magnitude > 4.0) && (this.Magnitude <= 5.25))
        {
            // GREEN->YELLOW
            r = (int)(255 * (this.Magnitude - 4.0) / 1.25); g = 255; b = 0;
        }
        else if ((this.Magnitude > 5.25) && (this.Magnitude <= 6.5))
        {
            // YELLOW->RED
            r = 255; g = 255 - (int)(255 * (this.Magnitude - 5.25) / 1.25); b = 0;
        }
        else if (this.Magnitude > 6.5)
        {
            // RED
            r = 255; g = 0; b = 0;
        }
        return new Color(r, g, b);
    }
    
    @Override
    public String toString() {
    	String nstStr = "";
    	String gapStr = "";
    	String dminStr = "";
    	String heStr = "";
    	String deStr = "";
    	String meStr = "";
    	String magNstStr = "";
    	if (Nst != -1) nstStr = "" + Nst;
    	if (Gap != -1) gapStr = "" + Gap;
    	if (Dmin != -1.0) dminStr = "" + Dmin;
    	if (HorizontalError != -1.0) heStr = "" + HorizontalError;
    	if (DepthError != -1) deStr = "" + DepthError;
    	if (MagError != -1) meStr = "" + MagError;
    	if (MagNst != -1) magNstStr = "" + MagNst;
    	
    	return "" + formatter.format(Time) + "," + Latitude + "," + Longitude + "," + Depth + "," + Magnitude 
    			+ "," + MagType + "," + nstStr + "," + gapStr + "," + dminStr + "," + Rms + "," + Net + "," + ID 
    			+ "," + Updated + "," + Place + "," + Type + "," + heStr + "," + deStr 
    			+ "," + meStr + "," + magNstStr + "," + Status + "," + LocationSource + "," + MagSource;    	
    	
    	/*return "" + formatter.format(Time) + "," + Latitude + "," + Longitude + "," + Depth + "," + Magnitude 
    			+ "," + MagType + "," + Nst + "," + Gap + "," + Dmin + "," + Rms + "," + Net + "," + ID 
    			+ "," + Updated + "," + Place + "," + Type + "," + HorizontalError + "," + DepthError 
    			+ "," + MagError + "," + MagNst + "," + Status + "," + LocationSource + "," + MagSource; 	
		*/
    }
    
    @Override
    public boolean equals(Object other)                             // Object.equals overriding
    {
    	if (other instanceof Earthquake) {
    		Earthquake that = (Earthquake) other;
    		return that.canEqual(this) 
    				&& this.Time.equals(that.Time) 
    				&& this.Latitude == that.Latitude
    				&& this.Longitude == that.Longitude
    	    	    && this.Depth == that.Depth
	    			&& this.Magnitude == that.Magnitude
  	    	    	&& this.MagType == that.MagType
    	    	    && this.ID == that.ID 
    				&& this.Place == that.Place;
    	}
    	return false;
    }

    @Override
    public int hashCode() {
    	return (Time.hashCode() + Double.hashCode(Magnitude) + Double.hashCode(Latitude) + Double.hashCode(Longitude) + Place.hashCode());
    }

    public boolean canEqual(Object other) {
    	return (other instanceof Earthquake);
    }
    
    public int compareTo(Earthquake other) {
    	return Time.compareTo(other.Time);
    }

	public static String[] columnNames() {
		return new String[] {"DateTime", "Latitude", "Longitude", "Depth", "Magnitude", "MagType", "ID", "PLACE"};
	}
}
