package EarthquakeLib;
import java.io.*;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class EarthquakeImporter {

    private static final String DATE_FORMAT_PATTERN = "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'";
	private static final SimpleDateFormat formatter = new SimpleDateFormat(DATE_FORMAT_PATTERN);
	private static String firstLine = null;
	private static BufferedReader br;

    // Load CSV
    public static List<Earthquake> loadCSV(String filename) {
    	List<Earthquake> quakeList = new ArrayList<Earthquake>();
    	
    	try {
    		File file = new File(filename);
    		if (file.exists()) {
    			br = new BufferedReader(new FileReader(filename));
                System.out.println("file import: " + filename);

                String line = "";
                String delimiter = ",";

                while ((line = br.readLine()) != null) {
                	
                	if (line.contains("time")) {
                		System.out.println(line);
                		firstLine = line; // set firstLine
                		continue;
                	}
                    // use comma as separator
                    String[] items = line.split(delimiter);
                    /*for(String s : items ) {
                    	System.out.println(s);
                    }*/
                    
                    // add Earthquake into List
                    Earthquake quake = parseEarthquake(items);
                    //System.out.println(quake);
                    quakeList.add(quake);       
                }
    		}
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } 

        return quakeList;
    }
    		
    public static Earthquake parseEarthquake(String[] items) {
    	Date time = tryParseDate(items[0]);
    	double lat = tryParseDouble(items[1]);
    	double lon = tryParseDouble(items[2]);
    	double depth = tryParseDouble(items[3]);
    	double mag = tryParseDouble(items[4]);
        String magType = items[5];
        int nst = tryParseInteger(items[6]);
        double gap = tryParseDouble(items[7]);
        double dmin = tryParseDouble(items[8]);
        double rms = tryParseDouble(items[9]);
        String net = items[10];
        String id = items[11];
        Date updated = tryParseDate(items[12]);
        
        String place = "", type = "", status = "", locSource = "", magSource = "";
        double he = -1.0, de = -1.0, me = -1.0;
        int magNst = -1;

        if (items.length == 22) {
        	place = items[13];
        	type = items[14];
        	he = tryParseDouble(items[15]);
        	de = tryParseDouble(items[16]);
        	me = tryParseDouble(items[17]);
        	magNst = tryParseInteger(items[18]);
        	status = items[19];
        	locSource = items[20];
        	magSource = items[21];
        } else if (items.length == 23) {
        	place = items[13] + "," + items[14];
        	type = items[15];
        	he = tryParseDouble(items[16]);
        	de = tryParseDouble(items[17]);
        	me = tryParseDouble(items[18]);
        	magNst = tryParseInteger(items[19]);
        	status = items[20];
        	locSource = items[21];
        	magSource = items[22];
        }
        // add Earthquake into List
        Earthquake quake = new Earthquake(time, lat, lon, depth, mag, magType, nst, gap, dmin, rms, net, id, updated, place, type, he, de, me, magNst, status, locSource, magSource);
        System.out.println(quake);
        return quake;
    }
    
    public static int tryParseInteger(String string) {
    	int num = -1;
		try {
			num = Integer.parseInt(string);			
		} catch (NumberFormatException e) {
			e.printStackTrace();			
		}
		return num;
	}

	public static double tryParseDouble(String string) {
		double num = -1.0;
		try {
			num = Double.parseDouble(string);			
		} catch (NumberFormatException e) {
			e.printStackTrace();			
		}
		return num;
	}

	public static Date tryParseDate(String string) {
		Date dateTime = null;	
		try {
			dateTime = formatter.parse(string);
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return dateTime;
	}

	// write CSV
    public static void writeCSV(String path, List<Earthquake> qList) {
    	FileWriter fw;
    	try {
			fw = new FileWriter(path);
			if (firstLine != null) fw.append(firstLine + "\n");
			for (Earthquake q : qList) {
				fw.append(q.toString() + "\n");
			}
			fw.flush();
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
}
