package EarthquakeLib;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;

public class EarthquakeManager {
	private List<Earthquake> quakeList = null;

	public EarthquakeManager() {
		quakeList = new ArrayList<Earthquake>();
	}

	public EarthquakeManager(List<Earthquake> quakeList) {
		this.quakeList = quakeList;
	}

	public void loadCSV(String filename) {
		this.quakeList = EarthquakeImporter.loadCSV(filename);
	}
	
	public void writeCSV(String filename) {
		EarthquakeImporter.writeCSV(filename, this.quakeList);
	}
	
	public List<Earthquake> getList() {
		return this.quakeList;
	}
	
	public int size() {
		return this.quakeList.size();
	}

	public boolean isEmpty() {
		return this.quakeList.isEmpty();
	}

	public Earthquake get(int index) {
		return this.quakeList.get(index);
	}

	public void set(int index, Earthquake q) {
		this.quakeList.set(index, q);
	}
	
	public void add(Earthquake q) {		
		this.quakeList.add(q);
	}

	public void addAll(List<Earthquake> qList) {		
		this.quakeList.addAll(qList);
	}

	public void removeAt(int index) {
		if (index >= 0 && index < size())	this.quakeList.remove(index);
	}

	public void sort(int index) {
		this.quakeList.sort(new EarthquakeComparator(index));
	}
	
	public void print() {
		this.quakeList.forEach(q -> System.out.println(q));
	}
	
	public List<Earthquake> findAllIf(Predicate<Earthquake> func) {
		return this.quakeList.stream().filter(func).collect(Collectors.toList());
	}
}





